///////////////////////////////////////////////////////////////////////////////
//
//	File    : $Id: tests.cpp 75 2006-05-12 08:40:41Z epocman $
//	Subject : IBPP, TEST1 program
//
///////////////////////////////////////////////////////////////////////////////
//
//	(C) Copyright 2000-2006 T.I.P. Group S.A. and the IBPP Team (www.ibpp.org)
//
//	The contents of this file are subject to the IBPP License (the "License");
//	you may not use this file except in compliance with the License.  You may
//	obtain a copy of the License at http://www.ibpp.org or in the 'license.txt'
//	file which must have been distributed along with this file.
//
//	This software, distributed under the License, is distributed on an "AS IS"
//	basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
//	License for the specific language governing rights and limitations
//	under the License.
//
///////////////////////////////////////////////////////////////////////////////
//
//	COMMENTS
//	* Tabulations should be set every four characters when editing this file.
//
///////////////////////////////////////////////////////////////////////////////
//
//	This file is NOT part of the IBPP core files.  It is NOT meant to be
//	compiled along with your own source code when you build a project using
//	IBPP.  This file is only a sample / test program to quickly verify the
//	major functionnalities of IBPP.  It is essentially used during development
//	of IBPP itself.  As such, you _obviously_ might have to adjust some
//	constants below to fit your specific configuration.
//
//	More specifically, please verify and adjust as you need it the constants
//	DbName, BkName, ServerName, UserName and Password before compiling this
//	test program.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef _MSC_VER
#pragma warning(disable: 4786 4996)
#ifndef _DEBUG
#pragma warning(disable: 4702)
#endif
#endif

#ifdef IBPP_WINDOWS
#include <windows.h>
#endif

#include "../core/ibpp.h"

#ifdef IBPP_UNIX
#include <unistd.h>
#define DeleteFile(x) unlink(x)
#define Sleep(x) usleep(1000 * x)
#endif

#ifdef HAS_HDRSTOP
#pragma hdrstop
#endif

#include <iostream>
#include <stdio.h>
#include <typeinfo>
#include <locale>

// Fix to famous MSVC 6 variable scope bug
#if defined(_MSC_VER) && (_MSC_VER < 1300)	// MSVC 6 should be < 1300
#define for if(true)for
#endif

//	The following database name (DbName) will be used during this test.
//	If existing, that DB will be deleted (assuming not used).
//	It will then be dynamically created, and various tests will run
//	while it is being populated and exercised.
//
//	The DB should grow to around 25 MB while being exercised.

#ifdef IBPP_UNIX
	const char* DbName = "~/test.fdb";
	const char* BkName = "~/test.fbk";
	const std::string ServerName = "localhost";
#else
	const char* DbName = "test.fdb";	// FDB extension (GDB is hacked by Windows Me/XP "System Restore")
	const char* BkName = "test.fbk";
	const std::string ServerName = "";	// Change to "" for local protocol / embedded
#endif

//	The tests use by default the well-known default of SYSDBA/masterkey
//	credentials. Do not forget to update them if required.

const std::string UserName = "SYSDBA";
const std::string Password = "masterkey";

class Test
{
	// Class 'Test' drives all the tests of this module.

	bool _Success;				// if true after all tests, everything succeeded
	int _WriteMode;				// 0 == default, 1 == speed, 2 == safety

public:
	void BlobTest1(void);
	void GenSBlob(IBPP::Database, IBPP::Transaction, IBPP::Blob, unsigned long, unsigned long);
	void RdSBlob(IBPP::Database, IBPP::Transaction, IBPP::Blob, unsigned long, unsigned long);
	void RunTests();
	int PrintResult();
	Test(int argc, char* argv[]);
	~Test();
};

int Test::PrintResult()
{
	if (_Success) printf(_("\n*** SUCCESS ***\nAll tests were 100%% successfull.\n"));
	else printf(_("\n*** FAILED ***\nSome tests failed, as indicated above.\n"));
	return _Success ? 0 : 1;
}

void Test::RunTests()
{
	int NextTest = 1;

	//IBPP::ClientLibSearchPaths("C:\\integral_90\\firebird\\bin");

	printf(_("\nIBPP Stream blob test (IBPP Version %d.%d.%d.%d)\n\n"),
		(IBPP::Version & 0xFF000000) >> 24,
		(IBPP::Version & 0x00FF0000) >> 16,
		(IBPP::Version & 0x0000FF00) >> 8,
		(IBPP::Version & 0x000000FF));

	for (;;)
	{
		try
		{
			switch (NextTest++)
			{
			    case 1 :    BlobTest1(); break; 
				default :	return;		// All tests have been run
			}
		}
		// You don't need to catch all these.
		// This is done only as a detailed sample case.
		// Often catching IBPP::Exception& is enough just to report the error string.
		// If you need access to the SQLCode and EngineCode obtained when the error
		// comes from the engine, catch(SQLException&) in addition or use RTTI.
		catch(IBPP::Exception& e)
		{
			_Success = false;
			std::cout << e.what() << "\n";
			IBPP::SQLException* pe = dynamic_cast<IBPP::SQLException*>(&e);
			if (pe == 0) std::cout<< _("Not an engine error\n");
			else std::cout<< "Engine Code : "<< pe->EngineCode()<< "\n";
		}
		catch(std::exception& e)
		{
			_Success = false;
			std::cout << e.what() << "\n";
			printf(_("Test %d --- FAILED\n")
				_("A std::exception '%s' occured !\n\n"),
					NextTest-1, typeid(e).name());
		}
		catch(...)
		{
			_Success = false;
			printf(_("Test %d --- FAILED\n")
				_("A system exception occured !\n\n"), NextTest-1);
		}
	}
}


// Stream blobs test
void Test::BlobTest1(void)
{
 int newid;
 IBPP::Database db1;
 
 db1 = IBPP::DatabaseFactory(ServerName, DbName, UserName, Password,
		"", "WIN1251", "PAGE_SIZE 8192 DEFAULT CHARACTER SET WIN1251");
 // create table
 try
 {
  db1->Create(3);		// 3 is the dialect of the database (could have been 1)
  db1->Connect(); 
  IBPP::Transaction tr1 = IBPP::TransactionFactory(db1,
							                       IBPP::amWrite,
												   IBPP::ilConcurrency,
												   IBPP::lrWait);
  IBPP::Statement st1 = IBPP::StatementFactory(db1, tr1);
  tr1->Start();
  st1->ExecuteImmediate("CREATE TABLE TEST("
						"WRITE_START TIMESTAMP, "
						"WRITE_END TIME, "
						"COMMIT_END TIME, "
						"B BLOB SUB_TYPE 0, "
						"ID INTEGER NOT NULL, "
						"PRIMARY KEY (ID))");
  st1->ExecuteImmediate("CREATE GENERATOR IDGEN");
  tr1->CommitRetain();
  db1->Disconnect();
 }
 
 catch(std::exception& e)
 {
/*
  _Success = false;
  std::cout << e.what() << "\n";
  printf(_("FAILED\nA std::exception '%s' occured !\n\n"), typeid(e).name()); 
*/
 }
 catch (...)
 {
  if (db1->Connected())
	  db1->Disconnect();
 }
 try
 { 
  {
   // write blob
   IBPP::Timestamp tstp;
   tstp.Now();
   db1->Connect(); 
   IBPP::Transaction tr1 = IBPP::TransactionFactory(db1,
							                       IBPP::amWrite,
												   IBPP::ilConcurrency,
												   IBPP::lrWait);
   tr1->Start();
   IBPP::Statement st1 = IBPP::StatementFactory(db1, tr1);
   IBPP::Blob bl1 = BlobFactory(db1, tr1);
   st1->Execute("SELECT GEN_ID(IDGEN, 1) from RDB$DATABASE");
   st1->Fetch();
   st1->Get(1,&newid);
   // write 1Mb data
   GenSBlob(db1, tr1, bl1, 262144*1, (unsigned int)newid);
   

   st1->Prepare("INSERT INTO TEST (WRITE_START, ID, B, WRITE_END) VALUES (?, ?, ?, 'NOW')");
   st1->Set(1,tstp);
   st1->Set(2,newid);
   st1->Set(3,bl1);

  // st1->Close();   
   st1->Execute();
   tr1->CommitRetain();
   // write end time
   tr1->Start();
   st1->Prepare("UPDATE TEST SET COMMIT_END = 'NOW' WHERE ID = ?");
   st1->Set(1,newid);
   st1->Execute();
   tr1->CommitRetain();
   db1->Disconnect();
  }
  {
   // read blobs
   db1->Connect();
   IBPP::Transaction tr1 = IBPP::TransactionFactory(db1,
							                       IBPP::amWrite,
												   IBPP::ilConcurrency,
												   IBPP::lrWait);
   tr1->Start();
   IBPP::Statement st1 = IBPP::StatementFactory(db1, tr1);
   IBPP::Blob bl1 = BlobFactory(db1, tr1);
   st1->Prepare("SELECT B FROM TEST WHERE ID=?");
   st1->Set(1,newid);
   st1->Execute();
   st1->Fetch();
   st1->Get(1,bl1);
   RdSBlob(db1, tr1, bl1, 262144*1, (unsigned int)newid);

   tr1->CommitRetain();
   db1->Disconnect();
  }
 }
 // You don't need to catch all these.
 // This is done only as a detailed sample case.
 // Often catching IBPP::Exception& is enough just to report the error string.
 // If you need access to the SQLCode and EngineCode obtained when the error
 // comes from the engine, catch(SQLException&) in addition or use RTTI.
 catch(IBPP::Exception& e)
 {
  _Success = false;
  std::cout << e.what() << "\n";
  IBPP::SQLException* pe = dynamic_cast<IBPP::SQLException*>(&e);
  if (pe == 0) std::cout<< _("Not an engine error\n");
  else std::cout<< "Engine Code : "<< pe->EngineCode()<< "\n";
 }
 catch(std::exception& e)
 {
  _Success = false;
  std::cout << e.what() << "\n";
  printf(_("FAILED\nA std::exception '%s' occured !\n\n"), typeid(e).name());
 }
 catch(...)
 {
  _Success = false;
  printf(_("FAILED\nA system exception occured !\n\n"));
 }
}




void Test::GenSBlob(IBPP::Database d, IBPP::Transaction t, IBPP::Blob b,
					unsigned long sz, unsigned long val)
{
 unsigned long temp1;
 unsigned int temp2;
 printf("Write blob\n");
 b->Create(IBPP_STREAM_BLOB | IBPP_BSTREAM);
 b->Info((int*)&temp2,(int*)&temp2,(int*)&temp2,(int*)&temp2);
 printf(" Blob type = %d\n", temp2);
 for (temp1=0; temp1<sz; temp1++)
 {
	temp2=temp1 ^ (val << 24);
 // b->Write((void*)&temp2, sizeof(temp2));
	 b->Putbx(temp2 &255);
	 b->Putbx((temp2>>8) &255);
	 b->Putbx((temp2>>16) &255);
	 b->Putbx((temp2>>24) &255);
 }
 //b->Cancel();
 b->Close();
}

void Test::RdSBlob(IBPP::Database d, IBPP::Transaction t, IBPP::Blob b,
					unsigned long sz, unsigned long val)
{
 char* tempointer;
 char  tp2;
 unsigned long temp1=0;
 long ofs, ofs2;   // seek offset
 int rdcnt;
 unsigned long rtot=0;
 printf("\nRead blob\n");
 b->Open(IBPP_BSTREAM);
 b->Info(&rdcnt,&rdcnt,&rdcnt,&rdcnt);
 printf(" Blob type = %d\n", rdcnt);
 printf(" seek test\n");
 ofs=65536-4;
 b->Seek(0, ofs, &ofs2);
 if (ofs!=ofs2)
	 printf(" seek error. ofs2=%d\n", ofs2);
 tempointer=(char*)&temp1;
 b->Getb();
 b->Getb();
 b->Seek(0, 0, &ofs2);
 printf(" verify blob data\n");
 rtot=0;
 tp2=0;
 do
 {
  rdcnt=b->Getb();
  if (rdcnt==EOF) break;
  tempointer[tp2]=rdcnt & 255;
  if (tp2 >=4)
  {
   tp2=0;
   if (*(unsigned long*)tempointer != (((rtot)>>2) ^ (val << 24)))
 		 printf(" blob read error: rd=%0d, val=%0d\n", *(unsigned long*)tempointer, ((rtot-1) >> 2)); 
  }
  rtot++;
 }
 while (1);
 printf (" read total %d\n\n", rtot);
 b->Close();
}



Test::Test(int argc, char* argv[])
{
	if (argc == 2 && argv[1] != 0 && strcmp(argv[1], "speed") == 0)
		_WriteMode = 1;
	else if (argc == 2 && argv[1] != 0 && strcmp(argv[1], "safety") == 0)
		_WriteMode = 2;
	else
		_WriteMode = 0;

	_Success = true;
}

Test::~Test()
{
}

int main(int argc, char* argv[])
{
 setlocale(LC_CTYPE,"russian.1251");
	Test T(argc, argv);

	if (! IBPP::CheckVersion(IBPP::Version))
	{
		printf(_("\nThis program got linked to an incompatible version of the IBPP source code.\n"
			"Can't execute safely.\n"));
		return 2;
	}

	T.RunTests();
system("PAUSE");
	return T.PrintResult();
}

//	Eof


